use clap::{arg, command, value_parser, ArgAction};
use std::path::PathBuf;

// https://docs.rs/clap/latest/clap/_tutorial/chapter_0/index.html

use sudoku::ksudoku::{load_ksudoku, save_ksudoku, Ksudoku};
use sudoku::sudoku::*;

fn main() {
    let args = command!()
        .arg(
            arg!(-f --file <FILE> "Filename to load")
                .required(false)
                .value_parser(value_parser!(PathBuf)),
        )
        .arg(
            arg!(
                -m --make ... "Make a new sudoku puzzle"
            )
            .action(ArgAction::SetTrue),
        )
        .arg(arg!(-b --brute ... "Brute-force solver").action(ArgAction::SetTrue))
        .arg(arg!(-d --debug ... "Debug output").action(ArgAction::SetTrue))
        .arg(
            arg!(-s --size <SIZE>... "Puzzle size")
                .required(false)
                .default_value("3")
                .value_parser(value_parser!(u8)),
        )
        .get_matches();

    let mut debug: bool = false;
    if args.get_flag("debug") {
        debug = true;
    }

    if args.get_flag("make") {
        let size: u8 = *args.get_one::<u8>("size").unwrap();
        let mut s = AnyBoard::new(size);

        /*
        s.make();
        let mut p = s.clone();

        // Ok, get working on the puzzle.
        for _x in 0..81 {
            while p.remove() {
                print!("-");
            }
        }
        println!("");

        p.display();
        println!("Solution:");
        s.display();
        if let Some(filename) = args.get_one::<PathBuf>("file") {
            let mut ksudo = Ksudoku::default();
            ksudo.puzzle = p.save_to_tld('b', '_');
            ksudo.solution = s.save_to_tld('b', '_');
            ksudo.order = 9; // Defaults currently 3x3
            let r = save_ksudoku(filename.to_path_buf(), &ksudo);
            if r.is_ok() {
                println!("Saved!");
            }
        }
        */
        return;
    }

    if let Some(filename) = args.get_one::<PathBuf>("file") {
        let result = load_ksudoku(filename.to_path_buf());
        if result.is_err() {
            println!("Failed to load file: {}", filename.display());
            // Safe to unrap error.
            println!("{:?}", result.unwrap_err());
            return;
        }

        let puzzle = result.unwrap();

        if puzzle.specific_type != "Plain" || puzzle.puzzle_type != "sudoku" {
            panic!(
                "We support Plain/sudoku not {}/{}",
                puzzle.specific_type, puzzle.puzzle_type
            );
        }

        let size = (puzzle.order as f64).sqrt() as u8;

        if size < 3 || size > 5 {
            panic!("We only support sizes 3, 4, and 5 (not {})", size);
        }

        let mut s = AnyBoard::new(size);
        let mut solution = AnyBoard::new(size);

        // Ksudoku is stored TLD.
        let result = s.load_from_tld('b', '_', &puzzle.puzzle);
        if result.is_ok() {
        s.display();
        } else {
            panic!("Failed loading puzzle: {}", result.unwrap_err());
        }

        let result = solution.load_from_tld('b', '_', &puzzle.solution);
        if result.is_err() {
            panic!("Failed loading result: {}", result.unwrap_err());
        }

        if args.get_flag("brute") {
            let (found, solutions) = s.brute_force_solver(2);
            println!("Solutions: {}", found);
            if found != 0 {
                if solutions[0] != solution {
                    println!("The solution doesn't match the one from the file:");
                    solution.display();
                }
            }
        } else {
            /*
            while s.solve(debug) {
                println!("Try it again...");
                if debug {
                    s.display();
                    s.display_possible();
                }
            }

            if !s.puzzle_complete() {
                if !debug {
                    println!("This is as far we could get with logic alone:");
                    s.display();
                }
                println!("Trying bruteforce.");
                println!("Bruteforce solutions: {}", s.bruteforce_solver());
            } else {
                if !debug {
                    s.display();
                }
            }
            */
        }
    }
}
